


/*
capture.cpp:
High Level Capture from Camera and Encoding of Video Streams.
Creation and deletion of group of threads for each video streamalong with the shared resources among them.
Synchronization among all the threads and shared resources. 

    Columbia Video Network Tool (CoVit)
    Copyright (C) 2009  Subhrendu Sarkar

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.



*/

#include <cstdlib>
#include <stdio.h>
#include <iostream>
#include <list>
#include <sys/time.h>
#include <time.h>

#include <ffmpeg/avcodec.h>
#include <ffmpeg/avformat.h>
#include <pthread.h>

#ifdef __unix
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <errno.h>
#include <sys/time.h>  /* struct timeval, gettimeofday() */
#include <sys/param.h> /* Might declare MAXHOSTNAMELEN */
#include <netdb.h>     /* Might declare MAXHOSTNAMELEN */
#include <unistd.h>    /* Might declare gethostname() */
#endif
using namespace std;
extern "C" {
#include "rtp_api.h"
#include "rtp_highlevel.h"
}

#include "common_defines.h"
#include "covit.h"

int capthreadid = 0;
int sndthreadid = 0;
int rcvthreadid = 0;


pthread_cond_t MediaBuffer_Empty_cond[30];
pthread_cond_t MediaBuffer_Full_cond[30];
pthread_mutex_t MediaBuffer_mutex[30];
void *capthread(void *param);

list<capNode> capList;
list<sendNode> sendList;
list<recvNode> recvList;

pthread_mutex_t libavcodec_mutex =  PTHREAD_MUTEX_INITIALIZER;
pthread_mutex_t stop_mutex = PTHREAD_MUTEX_INITIALIZER;
pthread_cond_t stop_cond = PTHREAD_COND_INITIALIZER;

extern void *sendrtp(void *param);
extern void *recvrtp(void *param);
extern int initformat();
void delete_buffers(CMediaBuffer *mbuffer);

/*
Name: initcapNode
Input parameters: capNode *, int streamId
Output: 
0:  SUCESS
-1: FAILURE
Description:
Initializes a node of the linked list "capList"
A capNode consists of capture parameters.
*/
int initcapNode(capNode *capN, int streamId)
{
	capN->id = streamId;
	pthread_mutex_init(&capN->capthr_cond_mutex, NULL);
	pthread_cond_init(&capN->capthr_cond, NULL);

	pthread_mutex_init(&capN->capstop_cond_mutex, NULL);
	pthread_cond_init(&capN->capstop_cond, NULL);
	pthread_mutex_init(&capN->capstatus_mutex, NULL);
	capN->stopCapStatus = 0;
	capN->encodeFmtNode.encode_fmt = CODEC_ID_MPEG2VIDEO; 
	capN->encodeFmtNode.encode_width = 320;
	capN->encodeFmtNode.encode_height = 240;
	capN->encodeFmtNode.encode_bitrate = 768000;
	capN->encodeFmtNode.fps = 30;
	capN->encodeFmtNode.gopSize = 20;

	return 0;
};


/*
Name: initsendNode
Input parameters: sendNode *, int streamId
Output: 
0:  SUCESS
-1: FAILURE
Description:
Initializes a node of the linked list "sendList"
A sendNode consists of parameters related to streaming (sending packets).
*/
int initsendNode(sendNode *sendN,int streamId)
{
	sendN->id = streamId;
	pthread_mutex_init(&sendN->sendthr_cond_mutex, NULL);
	pthread_cond_init(&sendN->sendthr_cond, NULL);

	pthread_mutex_init(&sendN->sendstop_cond_mutex, NULL);
	pthread_cond_init(&sendN->sendstop_cond, NULL);
	pthread_mutex_init(&sendN->sendstatus_mutex, NULL);
	sendN->send_quit = 0;
	return 0;
};

/*
Name: initrecvNode
Input parameters: recvNode *, int streamId
Output: 
0:  SUCESS
-1: FAILURE
Description:
Initializes a node of the linked list "recvList"
A recvNode consists of parameters related to receiving sending packets.
*/
int initrecvNode(recvNode *recvN, int streamId)
{
	recvN->id = streamId;
	pthread_mutex_init(&recvN->recvthr_cond_mutex, NULL);
	pthread_cond_init(&recvN->recvthr_cond, NULL);

	pthread_mutex_init(&recvN->recvstop_cond_mutex, NULL);
	pthread_cond_init(&recvN->recvstop_cond, NULL);
	//pthread_mutex_init(&recvN->sendstatus_mutex, NULL);
	recvN->recv_quit = 0;

	recvN->decodeFmtNode.decode_fmt = CODEC_ID_MPEG2VIDEO; 
	recvN->decodeFmtNode.decode_width = 320;
	recvN->decodeFmtNode.decode_height = 240;

	return 0;
};

/*
Name: createSendStreamId
Input parameters: int streamId
Output: 
0:  SUCESS
-1: FAILURE
Description:
Creates a Video Sending Stream and sets the streamId to the input parameter. 
*/
int createSendStreamId(int id)
{
	list<capNode>::iterator p;
	for(p = capList.begin(); p!= capList.end(); ++p)
	{
		if((*p).id == id)
		{
			return -1;
		}
	}

	list<sendNode>::iterator pS;
	for(pS = sendList.begin(); pS!= sendList.end(); ++pS)
	{
		if((*pS).id == id)
		{
			return -1;
		}
	}

	capNode captemp;
	sendNode sendtemp;
	initcapNode(&captemp, id);
	initsendNode(&sendtemp, id);
	
	capList.push_back(captemp);
	sendList.push_back(sendtemp);
	return 0;
};


/*
Name: createRecvStreamId
Input parameters: int streamId
Output: 
0:  SUCESS
-1: FAILURE
Description:
Creates a Video Receiving Stream and sets the streamId to the input parameter. 
*/
int createRecvStreamId(int id)
{
	list<recvNode>::iterator p;
	for(p = recvList.begin(); p!= recvList.end(); ++p)
	{
		if((*p).id == id)
		{
			return -1;
		}
	}
	recvNode recvtemp;
	initrecvNode(&recvtemp, id);
	recvList.push_back(recvtemp);
	return 0;
};


/*
Name: setEncodingFormat
Input parameters: 
int streamId: Stream ID of the Stream whose encoding format is to be set.
char *codecfmt: pointer to the codecfmt, it can be "MPEG4" "MPEG2" "MPEG1"
int width: width of the encoded video
int ht: height of the encoded video
int bitrate: bitrate of the encoded video

Output: 
0:  SUCESS
-1: FAILURE

Description:
Sets the encoding format to the sending stream associated with the corresponding stream ID. 
*/
int setEncodingFormat(int streamId,char *codecfmt, int width, int ht, int bitrate, int framerate)
{
	list<capNode>::iterator p;
	for(p = capList.begin(); p!= capList.end(); ++p)
	{
		if((*p).id == streamId)
		{
			if(strcmp(codecfmt,"MPEG4") == 0)
			{
				(*p).encodeFmtNode.encode_fmt = CODEC_ID_MPEG4;
			}
			else if(strcmp(codecfmt,"MPEG2") == 0)
			{
				(*p).encodeFmtNode.encode_fmt = CODEC_ID_MPEG2VIDEO;

			}	
	
			else if(strcmp(codecfmt,"MPEG1") == 0)
			{
				(*p).encodeFmtNode.encode_fmt = CODEC_ID_MPEG1VIDEO;

			}
			else
				return -1;	
	
			if(width !=0)
				(*p).encodeFmtNode.encode_width = width;
			if(ht !=0)
				(*p).encodeFmtNode.encode_height = ht;
			if(bitrate !=0)
				(*p).encodeFmtNode.encode_bitrate = bitrate;
	    if(framerate !=0)
				(*p).encodeFmtNode.fps = framerate;
	
			return 0;
		}
	}
	return -1;

}


/*
Name: init_all
Input parameters: NONE 
Output: NONE
Description:
Initializes the ffmpeg codecs and the default formats
*/
void init_all()
{
	av_register_all();
	initformat();
};

/*
Name: CVideoCapture()
Input parameters: int id
Output: NONE
Description:
Constructor of the higher level video capture object.
It takes the stream id as input to associate itself correctly with the corresponding stream parameters.
*/
CVideoCapture::CVideoCapture(int threadid)
{
	ObjVidCap = NULL;
	m_stopStatusBase = 0;

	m_MediaBuffer = NULL;
	VideoEnc = NULL;
	m_screen = NULL;
	m_threadid = threadid;
};

/*
Name: ~CVideoCapture()
Input parameters: 
Output: NONE
Description:
Destructor of the higher level video capture object.
*/
CVideoCapture::~CVideoCapture()
{
	delete  ObjVidCap;
	delete_buffers(m_MediaBuffer);
};

/*
Name: CVideoCapture::setMediaBuffer
Input parameters: 
CCapThreadParams * : pointer to CCapThreadParams
Output: NONE
Description:
sets the pointers of the mediabuffer resources to the appropriate mediabuffer resources already allocated and shared with the sending thread. 
*/
void CVideoCapture::setMediaBuffer(CCapThreadParams *p)
{
	m_MediaBuffer = p->MediaBuffer;
	m_capstatus_mutex = p->capstatus_mutex;

	m_thrstop_mutex = p->thrstop_mutex;
	m_thrstop_cond = p->thrstop_cond;	
	
	ObjVidCap->m_thrstop_mutex = p->thrstop_mutex;
	ObjVidCap->m_thrstop_cond = p->thrstop_cond;	
	ObjVidCap->m_threadid = m_threadid;


	ObjVidCap->m_capstatus_mutex = p->capstatus_mutex;
	for(int i = 0; i < 30; i++)
	{
		m_MediaBuffer_Empty_cond[i] = p->MediaBuffer_Empty_cond[i] ;
		m_MediaBuffer_Full_cond[i] = p->MediaBuffer_Full_cond[i];
		m_MediaBuffer_mutex[i] = p->MediaBuffer_mutex[i] ;

	}

	m_delMediaBuffer_Empty_cond = p->m_delMediaBuffer_Empty_cond;
	m_delMediaBuffer_Empty_cond = p->m_delMediaBuffer_Full_cond;
	m_delMediaBuffer_mutex = p->m_delMediaBuffer_mutex;
};

/*
Name: CVideoCapture::Process_Raw_Video
Input parameters: 
int buffers_ready: no of buffers ready (not used) 
int PixFormat: Picture Format of the raw captured Video 
int bufsize: size of the buffer from the camera
int width: width of the video data captured
int height: height of the video data captured
unsigned char *data: pointer to the buffer containing the captured video
int tsinc_usec: increment in timestamp in millisecs 
Output: NONE
Description:
This method is called from the lower level video capture object. This method takes as input the raw video data and its size.
It encodes the video data according to the format associated with its streamId and puts the encoded buffer in the Mediabuffers shared with the sending thread.
It also sets the timestamp to the encoded mediabuffer to the timestamp it obtains from the lower level capture object.
*/
void CVideoCapture::Process_Raw_Video(int buffers_ready,int PixFormat, int bufsize,int width, int height, unsigned char *data, int tsinc_usec)
{
	static int first_fr = 0;
	static int frame_cnt = 0;

	AVFrame *pFrame, *pFrameYUV;
	int sz;
	pFrame = avcodec_alloc_frame();
	pFrameYUV = avcodec_alloc_frame();
	uint8_t *buf;
	int numbytes = avpicture_get_size(PIX_FMT_YUV420P,  width, height);
	buf = (uint8_t *)av_malloc(numbytes * sizeof(uint8_t));
		
	sz = avpicture_fill((AVPicture *)pFrameYUV, buf, PIX_FMT_YUV420P, width, height);
	sz = avpicture_fill((AVPicture *)pFrame, data, PIX_FMT_YUV422, width, height);
	
	// Convert the image into YUV format that SDL uses
	img_convert((AVPicture *)pFrameYUV, PIX_FMT_YUV420P,(AVPicture *)pFrame,  PIX_FMT_YUV422, width, height);
	
	{//encode video
		static AVCodec *codec;
		static AVCodecContext *ctx = NULL;
		AVFrame *picture;
		int out_size, out_buf_size;
		uint8_t *out_buf;
		
		//Initialize the encoder at the first time only
		if(!first_fr)
		{
			//pthread_mutex_lock(&libavcodec_mutex);
			codec = avcodec_find_encoder(m_encode_fmt);//CODEC_ID_MPEG1VIDEO);//CODEC_ID_MPEG1VIDEO);//CODEC_ID_MPEG4);//CODEC_ID_MPEG1VIDEO);//
			//pthread_mutex_unlock(&libavcodec_mutex);		
			if(!codec)
			{
				cout << "Codec Not Found" << endl;
			} 
			ctx = avcodec_alloc_context();
			ctx->bit_rate = m_encode_bitrate;
			ctx->width = width;
			ctx->height = height;
			ctx->time_base = (AVRational){1,m_fps};
			ctx->gop_size = m_gopSize;
			ctx->max_b_frames = 0;
			ctx->pix_fmt = PIX_FMT_YUV420P;
			pthread_mutex_lock(&libavcodec_mutex);
			if(avcodec_open(ctx, codec) < 0)
			{
				cout << "Enc Could not open Codec" << endl;
				exit(1);		
			}
			pthread_mutex_unlock(&libavcodec_mutex);
			first_fr = 1;
			frame_cnt = 0;
		}
		
		picture = avcodec_alloc_frame();		
		out_buf_size = 100000;
		out_buf = (uint8_t *)malloc(out_buf_size);

		//Encode the video buffer
		out_size = avcodec_encode_video(ctx, out_buf, out_buf_size, (AVFrame *)pFrameYUV);
			//pthread_mutex_unlock(&libavcodec_mutex);		
				int sz;
		//get the buffer from the MediaBuffer		
		char *databuf = m_MediaBuffer->getBuffer(&sz,NULL);
		
		//if the buffer is in use, wait for the MediaBuffer to be emptied
		if( databuf != NULL)		
		{
			pthread_mutex_lock(m_MediaBuffer_mutex[m_MediaBuffer->bufferCount]);			
			pthread_cond_wait(m_MediaBuffer_Empty_cond[m_MediaBuffer->bufferCount], m_MediaBuffer_mutex[m_MediaBuffer->bufferCount]);
			pthread_mutex_unlock(m_MediaBuffer_mutex[m_MediaBuffer->bufferCount]);
		}			
		
		//Lock the Mediabuffer and set the Mediabuffer with the encoded video data
		pthread_mutex_lock(m_MediaBuffer_mutex[m_MediaBuffer->bufferCount]);
		m_MediaBuffer->setBuffer(out_size, (char *)out_buf, !frame_cnt, tsinc_usec/1000);
		pthread_cond_signal(m_MediaBuffer_Full_cond[m_MediaBuffer->bufferCount]);	
		pthread_mutex_unlock(m_MediaBuffer_mutex[m_MediaBuffer->bufferCount]);

		frame_cnt++;
		if(frame_cnt == ctx->gop_size)
			frame_cnt = 0;
		
		m_MediaBuffer = m_MediaBuffer->ObjNextBuffer;		
		//pthread_mutex_unlock(m_MediaBuffer_mutex[m_MediaBuffer->bufferCount]);
		free(out_buf);
		
		av_free(picture);
		av_free(pFrame);
		av_free(pFrameYUV);
		av_free(buf);
	}

};	

/*
Name: CVideoCapture::SetSettings
Input parameters: 
int device_no: device_no of the camera 
Output: NONE
Description:
Opens the camera device associated with the device no and sets the formats to be associated with the Video Capture Object.
*/
void CVideoCapture::SetSettings(int device_no)
{
	ObjVidCap = new CUnicapVidCap((IVideoCapture *)this, m_threadid);
//	int no_devices = ObjVidCap->enumerate_devices();
	
	ObjVidCap->open_device(device_no);
	
	ObjVidCap->setformats();
};

/*
Name: CVideoCapture::getVideoFormat
Input parameters: 
Output: NONE
Description:
*/
void CVideoCapture::getVideoFormat(){};


/*
Name: CVideoCapture::setVideoFormat
Input parameters: 
Output: NONE
Description:
*/
void CVideoCapture::setVideoFormat(){};

/*
Name: CVideoCapture::getVideoDeviceProp
Input parameters: 
Output: NONE
Description:
*/
void CVideoCapture::getVideoDeviceProp(){};


/*
Name: CVideoCapture::start_capture
Input parameters: 
CCapThreadParams *: pointer to CCapThreadParams which consists of parameters associated with the video stream id
Output: NONE
Description:
sets the different encoder parameters to be used by this Video Capture Object and then calls the start_capture method on the lower level video capture object.
*/
void CVideoCapture::start_capture(CCapThreadParams *cparam)
{
	list<capNode>::iterator p;
	for(p = capList.begin(); p!= capList.end(); ++p)
	{
		if((*p).id == m_threadid)
		{
			m_encode_bitrate = (*p).encodeFmtNode.encode_bitrate;
			m_encode_fmt = (*p).encodeFmtNode.encode_fmt;
			m_fps = (*p).encodeFmtNode.fps;		
			m_gopSize = (*p).encodeFmtNode.gopSize;			
			break;
		}
	}

	ObjVidCap->start_capture(cparam->stopCapStatus, m_fps);
};


/*
Name: CVideoCapture::stop_capture
Input parameters: 
Output: NONE
Description:
*/
int CVideoCapture::stop_capture()
{
};

/*
Name: starting
Input parameters:
int state: 0:Capture and Send, 1:Receiving Stream
char *ip_address: IP address to send or receive
int port: port number to send stream or port number to receive stream.
int streamId: stream ID of the stream 
Output:
0: SUCCESS
-1: FAILURE
Description:
This function is called from the XML RPC server to create & initilize all the shared resources and subsequently create and run the different threads as required.
*/
int starting(int state, char *ip_address, int port, int streamId)
{
	pthread_t recvthr, capthr, sendthr;
	
	//capture and sending stream		
	if(state == 0)
	{
		//find the stream ID, if stream ID does not exist return -1
		int ret = 0;
		list<sendNode>::iterator pS;
		for(pS = sendList.begin(); pS!= sendList.end(); ++pS)
		{
			if((*pS).id == streamId)
			{
				ret = 1;
				break;
			}
		}
		if(ret == 0)
			return -1;

		ret = 0;
		list<capNode>::iterator p;
		for(p = capList.begin(); p!= capList.end(); ++p)
		{
			if((*p).id == streamId)
			{
				ret = 1;
				break;
			}
		}
		if(ret == 0)
			return -1;



		CMediaBuffer *mBuffer = NULL;	
		CMediaBuffer *ObjMediaBuffer = new CMediaBuffer(0);
		CMediaBuffer *head = ObjMediaBuffer;
		CMediaBuffer *media_buf = head;
		CSenderThreadParams *tSparam = (CSenderThreadParams *)malloc(sizeof(CSenderThreadParams)) ;
		CCapThreadParams *tCparam = (CCapThreadParams *)malloc(sizeof(CCapThreadParams)) ;
		

		pthread_cond_t *MediaBuffer_Empty_cond = (pthread_cond_t *)malloc(30 * sizeof(pthread_cond_t)) ;
		pthread_cond_t *MediaBuffer_Full_cond = (pthread_cond_t *)malloc(30 * sizeof(pthread_cond_t)) ;;
		pthread_mutex_t *MediaBuffer_mutex = (pthread_mutex_t *)malloc(30 * sizeof(pthread_mutex_t)) ;;;
		
		pthread_mutex_init(&MediaBuffer_mutex[0], NULL);
		pthread_cond_init(&MediaBuffer_Empty_cond[0], NULL);
		pthread_cond_init(&MediaBuffer_Full_cond[0], NULL);

		tCparam->MediaBuffer_mutex[0] = tSparam->MediaBuffer_mutex[0] = &MediaBuffer_mutex[0];
		tCparam->MediaBuffer_Empty_cond[0] = tSparam->MediaBuffer_Empty_cond[0] = &MediaBuffer_Empty_cond[0] ;
		tCparam->MediaBuffer_Full_cond[0] = tSparam->MediaBuffer_Full_cond[0]
						= &MediaBuffer_Full_cond[0];	
		
		tCparam->m_delMediaBuffer_Empty_cond = MediaBuffer_Empty_cond;
		tCparam->m_delMediaBuffer_Full_cond = MediaBuffer_Full_cond ;
		tCparam->m_delMediaBuffer_mutex = MediaBuffer_mutex;
		
		//create 30 mediabuffers which is used as shared resource between the sending thread and the capture thread.
		for(int i = 1; i < 30; i++)
		{
			CMediaBuffer *nxt_MediaBuffer =  new CMediaBuffer(i);
			nxt_MediaBuffer->setPrevBuffer(media_buf);
			media_buf->setNextBuffer(nxt_MediaBuffer);
			media_buf = nxt_MediaBuffer;
			pthread_mutex_init(&MediaBuffer_mutex[i], NULL);
			pthread_cond_init(&MediaBuffer_Empty_cond[i], NULL);
			pthread_cond_init(&MediaBuffer_Full_cond[i], NULL);

			tCparam->MediaBuffer_mutex[i] = tSparam->MediaBuffer_mutex[i] = &MediaBuffer_mutex[i];
			tCparam->MediaBuffer_Empty_cond[i] = tSparam->MediaBuffer_Empty_cond[i] = &MediaBuffer_Empty_cond[i] ;
			tCparam->MediaBuffer_Full_cond[i] = tSparam->MediaBuffer_Full_cond[i]
							= &MediaBuffer_Full_cond[i];				
		}	 
		media_buf->setNextBuffer(head);
		head->setPrevBuffer(media_buf);

		mBuffer = head;

		tSparam->MediaBuffer = mBuffer;
		tCparam->MediaBuffer = mBuffer;

		strcpy(tSparam->host, ip_address);
		tSparam->port = port; 
		tSparam->ttl = 1;
		
		for(p = capList.begin(); p!= capList.end(); ++p)
		{
			if((*p).id == streamId)
			{
				tCparam->thr_mutex = &((*p).capthr_cond_mutex);
				tCparam->thr_cond = &((*p).capthr_cond);	
				tCparam->capstatus_mutex = &((*p).capstatus_mutex); 
				tCparam->threadid = (*p).id;

				tCparam->thrstop_mutex  = &((*p).capstop_cond_mutex);
				tCparam->thrstop_cond  = &((*p).capstop_cond);
				tCparam->stopCapStatus = &((*p).stopCapStatus);
				break;

			}				
		}

		for(pS = sendList.begin(); pS!= sendList.end(); ++pS)
		{
			if((*pS).id == streamId)
			{
				tSparam->thr_mutex = &((*pS).sendthr_cond_mutex);
				tSparam->thr_cond = &((*pS).sendthr_cond);	
				tSparam->sendstatus_mutex = &((*pS).sendstatus_mutex); 
				tSparam->sendstop_cond_mutex = &((*pS).sendstop_cond_mutex);
				tSparam->sendstop_cond = &((*pS).sendstop_cond);
				tSparam->send_quit = &((*pS).send_quit); 
				tSparam->threadid = ((*pS).id);


				break;

			}				
		}
		pthread_attr_t thread_attribute;
		int status;
		status = pthread_attr_init(&thread_attribute);
		if(!status)
			status = pthread_attr_setdetachstate(&thread_attribute, PTHREAD_CREATE_DETACHED);

		int rct = pthread_create(&capthr,&thread_attribute, capthread, (void *)tCparam);
		int rc = pthread_create(&sendthr,&thread_attribute, sendrtp, (void *)tSparam);	
	
		streamId = capthreadid; 
		capthreadid++;
		sndthreadid++;


	}
	//receiving stream	
	else if(state == 1)
	{
		int ret = 0;
		list<recvNode>::iterator pS;
		for(pS = recvList.begin(); pS!= recvList.end(); ++pS)
		{
			if((*pS).id == streamId)
			{
				ret = 1;
				break;
			}
		}
		if(ret == 0)
			return -1;

		CRecvThreadParams *tRparam = (CRecvThreadParams *)malloc(sizeof(CRecvThreadParams));
		
		rcvthreadid = 0;		
		pthread_attr_t thread_attribute;
		int status;
		

		
		list<recvNode>::iterator p;
		for(p = recvList.begin(); p!= recvList.end(); ++p)
		{
			if((*p).id == streamId)
			{
				tRparam->thr_mutex = &((*p).recvthr_cond_mutex);
				tRparam->thr_cond = &((*p).recvthr_cond);
				tRparam->recvstop_mutex = &((*p).recvstop_cond_mutex);
				tRparam->recvstop_cond = &((*p).recvstop_cond);
				tRparam->threadid = (*p).id;
				tRparam->recv_quit = &((*p).recv_quit);;
				break;

			}				
		}
		rcvthreadid++;
		status = pthread_attr_init(&thread_attribute);
		if(!status)
			status = pthread_attr_setdetachstate(&thread_attribute, PTHREAD_CREATE_DETACHED);
		
		strcpy(tRparam->host, ip_address);
		tRparam->port = port;
		tRparam->ttl = 1;
		int rt = pthread_create(&recvthr,&thread_attribute, recvrtp, (void *)tRparam );	
	}
	cout << "exiting" << endl;	
	return streamId;
}


/*
Name: stopSend
Input parameters:
int id: ID of the sending stream which is to be stopped
Output:
0: SUCCESS
-1: FAILURE
Description:
This function is called to stop sending video associated with the corresponding video stream ID
*/
int stopSend(int threadid)
{
	list<sendNode>::iterator p;
	for(p = sendList.begin(); p!= sendList.end(); ++p)
	{
		if((*p).id == threadid)
		{
			pthread_mutex_lock(&((*p).sendstatus_mutex));
			(*p).send_quit = 1;
			pthread_mutex_unlock(&((*p).sendstatus_mutex));
			
			pthread_mutex_lock(&((*p).sendstop_cond_mutex));
			pthread_cond_wait( &((*p).sendstop_cond), &((*p).sendstop_cond_mutex) );
			pthread_mutex_unlock( &((*p).sendstop_cond_mutex));
			cout << "stopSend End" <<endl; 
			
			break;
		}
	}
	sendList.erase(p);
	cout << "stopSend Erase" <<endl;
	return 0;
};


/*
Name: delete_buffers
Input parameters:
CMediaBuffer *mbuffer: pointer to CMediaBuffer
Output:NONE
Description:
This function is called to delete the MediaBuffers. It deletes all the Mediabuffers linked together in the circular linked list
*/
void delete_buffers(CMediaBuffer *mbuffer)
{
	if(mbuffer->ObjNextBuffer->bufferCount != 1)
	{
		delete_buffers(mbuffer->ObjNextBuffer);
	}
	delete mbuffer->ObjNextBuffer ;

};


/*
Name: stopLocal
Input parameters:
int id: ID of the sending stream which is to be stopped
Output:
0: SUCCESS
-1: FAILURE
Description:
This function is called from the XML RPC server to stop sending video associated with the corresponding video stream ID and also stop capturing video from the camera with the associated stream ID.
*/
int stopLocal(int threadid)
{
	stopSend(threadid);
	list<capNode>::iterator p;
	for(p = capList.begin(); p!= capList.end(); ++p)
	{
		if((*p).id == threadid)
		{
			pthread_mutex_lock(&((*p).capstatus_mutex));
			(*p).stopCapStatus = 1;
			pthread_mutex_unlock(&((*p).capstatus_mutex));
			cout << "stopCap threadid " << threadid << " " << (*p).stopCapStatus << endl; 

			pthread_mutex_lock(&((*p).capstop_cond_mutex));
			pthread_cond_wait( &((*p).capstop_cond), &((*p).capstop_cond_mutex) );
			pthread_mutex_unlock( &((*p).capstop_cond_mutex));
			
			break;
		}
	}
	capList.erase(p);
	return 0;
};

/*
Name: stopSend
Input parameters:
int id: ID of the sending stream which is to be stopped
Output:
0: SUCCESS
-1: FAILURE
Description:
This function is called to XML RPC server to stop receiving video associated with the corresponding video stream ID
*/
int stopRecv(int threadid)
{
	list<recvNode>::iterator p;
	for(p = recvList.begin(); p!= recvList.end(); ++p)
	{
		if((*p).id == threadid)
		{
			pthread_mutex_lock(&((*p).recvstop_cond_mutex));
			(*p).recv_quit = 1;
			pthread_mutex_unlock(&((*p).recvstop_cond_mutex));
			cout << "stopRecv threadid " << threadid << " " << (*p).recv_quit << endl; 

			pthread_mutex_lock(&((*p).recvthr_cond_mutex));
			pthread_cond_wait( &((*p).recvthr_cond), &((*p).recvthr_cond_mutex) );
			pthread_mutex_unlock( &((*p).recvthr_cond_mutex));
			cout << "stopRecv End" <<endl; 
				
			break;
		}
	}
	recvList.erase(p);
	return 0;

};


/*
Name: capthread
Input parameters:
void *param: parameters
Output:NONE
Description:
Function for the Capture thread.
*/
void *capthread(void *param)
{
	CVideoCapture *VidCapture = NULL;
	CCapThreadParams *cparam = (CCapThreadParams *)param;
	//pthread_mutex_t *thr_mutex = cparam->thr_mutex; 
	//pthread_cond_t  *thr_cond = cparam->thr_cond; 
	
	//Creating the Video Capture Object, setting parameters, starting capture and stopping capture
	do{
		cout << "capthread" <<endl;
			
		if(VidCapture == NULL)		
			VidCapture = new CVideoCapture(cparam->threadid);
		else
		{
			VidCapture->stop_capture();
			delete VidCapture;
			VidCapture = NULL;  
			break;
		}
		VidCapture->SetSettings(0);
		
		VidCapture->setMediaBuffer(cparam);

		VidCapture->start_capture(cparam);
		VidCapture->stop_capture();
		
		delete VidCapture;
		VidCapture = NULL;  
	
	}while(0);

	//pthread_mutex_lock( thr_mutex  );
	//pthread_cond_signal( thr_cond );
	//pthread_mutex_unlock( thr_mutex );
	
	cout << "capthread exiting" << endl;	

	free(cparam->m_delMediaBuffer_Empty_cond);
	free(cparam->m_delMediaBuffer_Full_cond);
	free(cparam->m_delMediaBuffer_mutex);
	

	free(cparam);
	pthread_exit(NULL);

};
	
