/*
    data structure definitions used across the source files.
    Columbia Video Network Tool (CoVit)
    Copyright (C) 2009  Subhrendu Sarkar

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/

#ifndef INCLUDE_COMMON_DEFINES_H
#define INCLUDE_COMMON_DEFINES_H
#define BUFFERS 6 //1 //6
#define RET_SUCCESS 0
#define RET_FAILURE 1 
#include <ffmpeg/avcodec.h>
#include <ffmpeg/avformat.h>




enum Covit_PixelFmt {
	COVIT_PIX_FMT_NONE= -1, 
	COVIT_PIX_FMT_YUV420,
	COVIT_PIX_FMT_YUV422,
	COVIT_PIX_FMT_RGB32,
	COVIT_PIX_FMT_RGB24,
	COVIT_PIX_FMT_BGR
};

#define COVIT_PIX_FMT_YUYV COVIT_PIX_FMT_YUV422

//encode format
typedef struct encodeformat
{
	//codec type - MPEG4, MPEG2, MPEG1, etc
  CodecID encode_fmt;
  //encoder video width 
	int encode_width;
  //encoder video height
	int encode_height;
  //encoder bitrate
	int encode_bitrate;
  //encoder frame rate
	int fps;
  //encoder GOP size
	int gopSize;
};

//decode format
typedef struct decodeformat
{
	//codec type - MPEG4, MPEG2, MPEG1, etc
  CodecID decode_fmt;
  //decode video width
	int decode_width;
  //decode video height
	int decode_height;
};

//node of capList 
typedef struct capNode
{
  //capture or send video stream identifier
	int id;
  //capture thread mutex
	pthread_mutex_t capthr_cond_mutex;
  //capture thread conditional for signaling
	pthread_cond_t  capthr_cond;
	//capture thread stop mutex
  pthread_mutex_t capstop_cond_mutex;
	//capture thread conditional for signaling
  pthread_cond_t  capstop_cond;
	//capture thread status mutex
  pthread_mutex_t capstatus_mutex;
	//capture thread stop status
  int stopCapStatus;
  //encode format
	encodeformat encodeFmtNode;
};

//node of sendList
typedef struct sendNode
{
  //send or capture video stream identifier
	int id;
  //sender thread mutex
	pthread_mutex_t sendthr_cond_mutex;
	//sender thread conditional for signaling
  pthread_cond_t  sendthr_cond;
  //sender thread stop mutex
	pthread_mutex_t sendstop_cond_mutex;
	//sender thread stop conditional for signaling
  pthread_cond_t sendstop_cond;
  //sender thread status mutex
	pthread_mutex_t sendstatus_mutex;
  //variable for sender thread quit 
  int send_quit;	 
};

//node of recvList
typedef struct recvNode
{
  //receive video stream identifier
	int id;
  //receiver thread mutex
	pthread_mutex_t recvthr_cond_mutex;
  //receiver thread conditional for signaling
	pthread_cond_t  recvthr_cond;

	//receiver thread stop mutex
  pthread_mutex_t recvstop_cond_mutex;
	//receiver thread stop conditional for signaling
  pthread_cond_t  recvstop_cond;
  //variable to quit receiver thread associated with the video stream identifier
	int recv_quit;
  //decode format
	decodeformat decodeFmtNode;
};




#endif
