/*
    Copyright (C) 2009  Subhrendu Sarkar

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/

#include <iostream>
/*
This is a class that represents a Media buffer and contains pointers to the next and previous Media Buffers. 
*/
class CMediaBuffer
{
private	:
  //the buffer which holds the media data
	char *dataBuffer;
	//the timestamp increment associated with the media buffer in millisecs
	int tsinc_msec;	
  //size of the media buffer
	int buffer_size;
	
public :
  //count of the buffer out of the 30 media buffers
	int bufferCount;
	//pointer to next Media Buffer
	CMediaBuffer *ObjNextBuffer;
  //pointer to previous Media Buffer
	CMediaBuffer *ObjPrevBuffer;
  //constructor of Media Buffer, it takes as input an integer which is the count of the buffer
	CMediaBuffer(int x)
	{
		bufferCount = x;
		dataBuffer = NULL;
		ObjNextBuffer = NULL;
		ObjPrevBuffer = NULL;
	};
  //returns the pointer to the media buffer data and sets the size and time stamp incremenet associated with the media buffer
	char *getBuffer(int *size,int *timestampInc)
	{
		*size = buffer_size; 
		
		if(timestampInc != NULL)
			*timestampInc = tsinc_msec; 
		return dataBuffer; 
	};
  //free the media bufffer data
	void freeBuffer()
	{
		free(dataBuffer);
		dataBuffer = NULL;
	};	
  //allocates the media buffer data and copies the data to media buffer data and the time stamp increment; returns 0 on SUCCESS, -1 on FAILURE
	int setBuffer(int size, char *buf, int ts_msec)
	{
		dataBuffer = (char *)malloc(size * sizeof(char));
		if(dataBuffer == NULL)
		{
			cout << "Cannot allocate Memory " << endl;
			return -1;		
		}
		memcpy(dataBuffer,buf, size);
		buffer_size = size;
		tsinc_msec = ts_msec;
		return 0;

	};
  //sets the next Media Buffer
	void setNextBuffer(CMediaBuffer *nxtbuf)
	{
		ObjNextBuffer = nxtbuf;	
	};
  //sets the previous Media Buffer
	void setPrevBuffer(CMediaBuffer *prevbuf)
	{
		ObjPrevBuffer = prevbuf;
		
	};
};

