/*
    Copyright (C) 2009  Subhrendu Sarkar

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/
#ifndef INCLUDE_UNICAPVIDCAP_H
#define INCLUDE_UNICAPVIDCAP_H
#include <unicap.h>
#include <unicap_status.h>
#include "interfaces.h"
//#include "common_defines.h"

/*
Class for Camera Device 
*/
class CCameraDevice
{
public:
  //singleton Camera Device object; returns the single instance of the object
	static CCameraDevice* Instance();
	//constructor of camera device class
	CCameraDevice();
  //destructor
	virtual ~CCameraDevice();
  // returns the number of camera devices in the system
	int getNumberofCams();
  //sets the camera handle for the device number specified
	int getCamHandle(unicap_handle_t *handle, int device_no = 0);
  //closes the camera handle for the device number
	int closeCamHandle(int device_no);
  //destroy the instance for the CCameraDevice
	void destroy();
  //sets the capture format for the device number
	int setformats(int device_no, unicap_format_t *format);
	//starts capture for the device number
  int start_capture(int device_no);
  //stops capture
	void stop_capture();
  //initializes the camera devices 
	int initCamera();
  //deinitializes the camera devices 
	void deinitCams();
  //the single instance of CCameraDevice 
	static CCameraDevice* m_CameraInstance;
  //reference count of the CCameraDevice object
	int m_refCount;
private:
  //number of Camera devices in the system
	int m_NumCams;	
  //buffer to hold captured video from the devices
	unicap_data_buffer_t **m_buffers;

  //array of variables to indicate capture state for each camera device; 0:device not started 1:captured started
	int *m_start_capture;
  //array of reference counts for each capture device
	int *m_hndRefCount;
  //array of camera handles for each device
	unicap_handle_t *m_handle;
	//array of camera device
  unicap_device_t *m_device;
  //array of formats for each camera device
	unicap_format_t *m_format_spec;
	unicap_format_t *m_format;

};

//class to act as an interface with camera device. Each instance of the  CUnicapVidCap object is associated with one camera device. 
class CUnicapVidCap
{
public :
	//~CUnicapVidCap(){};
//	static CUnicapVidCap* Instance(IVideoCapture *, int threadid);
  //constructor
	CUnicapVidCap(IVideoCapture *, int streamid);
	//destructor
  virtual ~CUnicapVidCap();	

//	void destroy();
  //open device with the device number
	void open_device(int device_no);
  //returns the number of devices in the system 
	int enumerate_devices();
  //sets formats for the camera devices
	void setformats();
  //start capture; takes as input the address of the stop_status which is used to stop capture externally, fps to indicate the encoding frame rate
	void start_capture(int *stop_status, int fps);
  //stop capture
	void stop_capture(); 

  //capture status mutex
	pthread_mutex_t *m_capstatus_mutex;
  //thread stop mutex
	pthread_mutex_t *m_thrstop_mutex;
  //thread stop conditional for signaling purposes
	pthread_cond_t *m_thrstop_cond;
	//thread id associated with the object
  int m_threadid;	
	
private :
  //device number associated with the object
	int m_device_no;
  //handle of the camera
	unicap_handle_t m_handle;
  //device structure for the camera
	unicap_device_t m_device;
  //format for the camera
	unicap_format_t m_format_spec;
	unicap_format_t m_format;
  //array of buffers to hold the captured data from the camera 
	unicap_data_buffer_t m_buffers[BUFFERS];
	unicap_data_buffer_t *m_returned_buffer;
	
	//static CUnicapVidCap* m_instance; 
	int m_PixFmt;
	//CCameraDevice object
	CCameraDevice *m_Cam;
	IVideoCapture *BaseVidCapture;
};
#endif
